<div id="id_{{ form_field.html_name }}_container" class="control-group {% if css_classes %}{{ css_classes }}{% endif %} {% if form_field.errors %}error{% endif %}">
    <div class="input">
        <label>
            {{ form_field.label }} {% if form_field.field.required %}<span class="req">*</span>{% endif %}
            {% if help_text %}
                <span class="help-inline">{{ help_text }}</span>
            {% endif %}
            {% if form_field.errors %}
                <span class="help-block">{{ form_field.errors }}</span>
            {% endif %}
        </label>
        {{ form_field }}
        
    <div/>
</div>