<div id="id_{{ form_field.html_name }}_container" class="control-group {% if css_classes %}{{ css_classes }}{% endif %} {% if form_field.errors %}error{% endif %}">
    <label class="checkbox controls" for="id_{{ form_field.html_name }}">
        {{ form_field }}
        {{ form_field.label }} {% if form_field.field.required %}<span class="req">*</span>{% endif %}
        {% if help_text %}
            <span class="help-inline">{{ help_text }}</span>
        {% endif %}
        {% if form_field.errors %}
            <span class="help-block">{{ form_field.errors }}</span>
        {% endif %}
    </label>
</div>