<div id="id_{{ form_field.html_name }}_container" class="control-group{% if css_classes %} {{ css_classes }}{% endif %} {% if form_field.errors %}error{% endif %}">
    <label class="control-label" for="id_{{ form_field.html_name }}">
        {{ form_field.label }}{% if form_field.field.required %} <span class="req">*</span>{% endif %}
    </label>
    <div class="controls">
        {{ form_field }}
        {% if form_field.help_text %}
            <span class="help-inline">{{ form_field.help_text }}</span>
        {% endif %}
        {% if form_field.errors %}
            <span class="help-block">{{ form_field.errors }}</span>
        {% endif %}
    </div>
</div>